//+------------------------------------------------------------------+
//|                                             AA_Stoch_100_Sym.mq4 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window


sinput int nbrOfBars = 5000;

bool bStart = true;
bool SellTrade, BuyTrade, ActiveTrade;
double startPrice, endPrice;


bool onlyBuyTrades = false;
bool onlySellTrades = false;

double myDigits, myPoint, myMultiplier;

string preamble = "AA_Stoch8over21_";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
  bStart = true;
  SellTrade = false;
  BuyTrade = false;
  ActiveTrade = false;
  myDigits = MarketInfo(Symbol(),MODE_DIGITS);
  myPoint = MarketInfo(Symbol(),MODE_POINT);
  if( myDigits == 3 || myDigits == 5 ) myPoint = myPoint * 10;
  myMultiplier = 1.0 / myPoint;
  string s, name;
  s = preamble;
  for (int i = ObjectsTotal() - 1; i >= 0; i--)
    {
        name = ObjectName(i);
        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
    } 
//---
   return(INIT_SUCCEEDED);
  }
  
void OnDeinit(const int reason)
  {
//--- destroy timer
   EventKillTimer();
   
   // delete objects
    string name;
    string s = preamble;
 
    for (int i = ObjectsTotal() - 1; i >= 0; i--)
    {
        name = ObjectName(i);
        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
    }
      
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   double sum = 0;
   int winCounter = 0;
   int lossCounter = 0;
   int counter = 0;
   string linename ;
   double tmpvalue_1, tmpvalue_2, tmpvalue_3, tmpvalue_4;
   
   if (!bStart)
   {
      return(rates_total);
   }
   
   if (bStart)
   {
      bStart = false;
      
      for (int u = nbrOfBars; u > 0; u--)
      {
         // check for buy or sell trades
         if (iStochastic(Symbol(), Period(),100,10,10,MODE_SMA,1,MODE_MAIN,u) > 50)
         {
            onlyBuyTrades = true;
            onlySellTrades = false;
         }
         else
         {
            onlyBuyTrades = false;
            onlySellTrades = true;
         }
         
         tmpvalue_1 = iStochastic(Symbol(), Period(),21,10,10,MODE_SMA, STO_CLOSECLOSE,MODE_MAIN,u);
         tmpvalue_2 = iStochastic(Symbol(), Period(),21,10,10,MODE_SMA, STO_CLOSECLOSE,MODE_MAIN,u+1);
         tmpvalue_3 = iStochastic(Symbol(), Period(),8,3,3,MODE_SMA, STO_CLOSECLOSE,MODE_MAIN,u);
         tmpvalue_4 = iStochastic(Symbol(), Period(),8,3,3,MODE_SMA, STO_CLOSECLOSE,MODE_MAIN,u+1);
         
         if (onlyBuyTrades)
         {
            if (!ActiveTrade)
            {
               if (tmpvalue_3 > tmpvalue_1 && tmpvalue_4 < tmpvalue_2)
               {
                  startPrice = iClose(Symbol(), Period(),u);
                  ActiveTrade = true; 
                  BuyTrade = true;
                  linename = preamble + IntegerToString(counter);
                  counter++;
                  ObjectCreate(0,linename,OBJ_VLINE,0,iTime(Symbol(), Period(),u),0);
                  ObjectSetInteger(0,linename,OBJPROP_COLOR,clrLime);
               }
            }
         }
         
         if (onlySellTrades)
         {
            if (!ActiveTrade)
            {
               if (tmpvalue_3 < tmpvalue_1 && tmpvalue_4 > tmpvalue_2)
               {
                  startPrice = iClose(Symbol(), Period(),u);
                  ActiveTrade = true; 
                  SellTrade = true;
                  linename = preamble + IntegerToString(counter);
                  counter++;
                  ObjectCreate(0,linename,OBJ_VLINE,0,iTime(Symbol(), Period(),u),0);
                  ObjectSetInteger(0,linename,OBJPROP_COLOR,clrLime);
               }
            }
         }
         
         if (BuyTrade)
         {
            if (tmpvalue_3 < tmpvalue_1 && tmpvalue_4 > tmpvalue_2)
            {
               endPrice = iClose(Symbol(), Period(),u);
               if (endPrice > startPrice) 
               { 
                  winCounter++;
               }
               else
               {
                  lossCounter++;
               }
               BuyTrade = false;
               ActiveTrade = false;
               sum = sum + (endPrice - startPrice) * myMultiplier;
               linename = preamble + IntegerToString(counter);
               counter++;
               ObjectCreate(0,linename,OBJ_VLINE,0,iTime(Symbol(), Period(),u),0);
               ObjectSetInteger(0,linename,OBJPROP_COLOR,clrRed);
            
            }
         
         }
         
         if (SellTrade)
         {
            if (tmpvalue_3 > tmpvalue_1 && tmpvalue_4 < tmpvalue_2)
            {
            
               endPrice = iClose(Symbol(), Period(),u);
               if (endPrice < startPrice) 
               { 
                  winCounter++;
               }
               else
               {
                  lossCounter++;
               }
               SellTrade = false;
               ActiveTrade = false;
               sum = sum - (endPrice - startPrice) * myMultiplier;
               linename = preamble + IntegerToString(counter);
               counter++;
               ObjectCreate(0,linename,OBJ_VLINE,0,iTime(Symbol(), Period(),u),0);
               ObjectSetInteger(0,linename,OBJPROP_COLOR,clrRed);
            }
         }
      }
   
   
   }
   Alert("Winners: " + winCounter);
   Alert("Losses: " + lossCounter);
   Alert("Pips: " + sum);
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   
  }
//+------------------------------------------------------------------+
